#!/bin/sh
#
# This script, usually run once during system startup from /etc/inittab,
# Checks for an existane of a panicdump in devices such as MTD.
# If found, and if newer than the one found under /core_files/panic/
# then it copies the panicdump from MTD to destination directory.
#

BIN=/fabos/bin/
PD_SUCCESS=0

# core files and now pd location. This is a directory name
CORE_FILES_DIR=/core_files/	# The place where we store panicdump
				# which is same for core files
				# see savecore also, where this is used

PD_DIR=$CORE_FILES_DIR/panic/

# pd Control file. Will contain all the control date related to pd.
# It contains the time-stamp of pd.
# Note: this time-stamp is the time retrived from MTD and NOT the
# current time, when we saved this
PD_CTRL_FILE=/etc/.pd_data	# We will NOT store this in $CORE_FILES
				# This is because the existance is NOT
				# gauranteed and an user may choose to
				# delete $CORE_FILES_DIR. But we need
				# this time stamp
PD_TEMP_ERRLOG_FILE=/tmp/pd_temp_errlog

MISC_FILE=/core_files/.misc_file  # if you change this, also change
                                # corresponding entry in pdul_misc_pre.sh and
                                # pdul_misc_post.sh and pdcheck.sh

MAX_PD_FILES=10	# At this time we will store up to 10 panic dumps

#
# function used to count the number of files in panic 
# # Check if there are too many dump files are around

checkIfDumpExceeded() {
	pd_file_count=`/bin/ls -l ${PD_DIR}/core.* 2>/dev/null | /usr/bin/wc -l`
	if [ $pd_file_count -gt $MAX_PD_FILES ]
	then
		return 1
	fi
	return 0
}
#
#	Function used to check for a presence of a dumpfile
#	if new pd found, record the date in $mtd_ts and return 1
#	returns 0 otherwise
#
checkForNewDump() {
	# just checking if pdcp works
	$BIN/pdcp /dev/panicdump 1>/dev/null 2>/dev/null
	rv=$?
	if [ $rv -ne $PD_SUCCESS ]
	then
		# well, our MAGIC# is not found
		# we will excuse if /dev/panicdump is of "\000" NULLs for 10K.
		# we resorted to 10k limit because of time limit to write NULLs

		nonNulls=`/bin/dd if=/dev/panicdump bs=1024 count=10 2>/dev/null | /usr/bin/tr "\000" -d | wc -c`
		if [ "$nonNulls" -eq "0" ]
		then
			# this is a "clean" flash, so let us just return
			return 0
		fi

		# Some error condition, like,
		#	either no valid pd exists in MTD
		#	or     there was some error condition in MTD
		#   This is usually the case when we run the new pd code
		#   on a cp where pd has not been implemented, such as
		#   upgrade from 4.0.0 to 4.1
		# Anyways, we will assume NO new pd and clean up pd partition

		/bin/dd if=/dev/zero of=/dev/panicdump bs=1024 count=10 2>/dev/null
		printf "Info: panic dump partition has been initialized!\n"
		printf "Info: panic dump partition has been initialized!\n" \
					>> ${PD_TEMP_ERRLOG_FILE}
		return 0
	fi
	# now that there is some valid pd in MTD
	# so, let us check if we have seen this already...
	# by comparing it with time stamp in CF
	mtd_ts_full=`$BIN/pdcp /dev/panicdump 2>/dev/null`
	mtd_ts=`echo $mtd_ts_full | /usr/bin/cut -f1 -d' '`
	[ -z "$mtd_ts" ] && mtd_ts=mtd_ts

	# We will remove comments along with resulting blank lines
	cf_ts_full=`/bin/sed -e \
		's/#.*//;/^[ \t]*$/d' \
		$PD_CTRL_FILE 2>/dev/null`
	cf_ts=`echo $cf_ts_full | /usr/bin/cut -f1 -d' '`
	[ -z "$cf_ts" ] && cf_ts=cf_ts

	if [ $mtd_ts == $cf_ts ]
	then
		# We have already seen this...
		return 0
	fi
	# We found a different pd in MTD!
	printf "pdcheck: info: found new pd in MTD: mtd_ts=%s, cf_ts=%s\n" \
		"$mtd_ts_full" "$cf_ts_full"
	printf "pdcheck: info: found new pd in MTD: mtd_ts=%s, cf_ts=%s\n" \
		"$mtd_ts_full" "$cf_ts_full"  \
                                        >> ${PD_TEMP_ERRLOG_FILE}

	return 1
}

#
# main starts here
# let us see if panicdump is chkconfig'd off
chkconfig panicdump
if [ $? -eq 1 ]
then
        # 
        # we will turn off the panic dump
        # Please see panicdump.h and panicdump.c for more details on
        # these strings used (like ioctl)
        #
        printf "PD_IOCTL1" > /proc/panicdump
fi

#
# Let us first calculate the date -- This will be seconds like time(NULL)
#

DATE=`/bin/date +"%s"`

# Let us create the panic directory, if this does not exists already
if [ ! -d $PD_DIR ]
then
	# delet anything that is not a directoy...
	rm -f $PD_DIR 1>/dev/null 2>/dev/null
	# and create a diretory
	mkdir $PD_DIR
fi

# Let us create the date file, if this does not exists
if [ ! -f $PD_CTRL_FILE ]
then
	# The file does not exists so, create it!
	printf "0\n" > $PD_CTRL_FILE
	printf "pdcheck: info: Created New pd control file\n"
	printf "pdcheck: info: Created New pd control file\n"  \
                                        >> ${PD_TEMP_ERRLOG_FILE}

fi

checkForNewDump # Checking for new dump
if [ $? -eq 0 ]
then
	# no new dump, so let us just log this and exit
	exit 0
fi
checkIfDumpExceeded # Check if there are too many dump files are around
if [ $? -eq 1 ]
then
	# There ARE too many files left out so let us just skip this
	# We will anyways get this when dumps are cleared
	printf "pdcheck: WARNING: panicdump: number of files maxed out\n"
	printf "pdcheck: WARNING: panicdump: number of files maxed out\n" \
                                        >> ${PD_TEMP_ERRLOG_FILE}
	exit 0
fi
#
# new dump found, so let us save it and also save the time after
# successful completion of the whole dump
#
$BIN/pdcp /dev/panicdump ${PD_DIR}/core.pd$DATE
rv=$?
if [ $rv -ne $PD_SUCCESS ]
then
	printf "pdcheck: pdcp failed to copy the files (err = %d)\n" $rv
	printf "pdcheck: pdcp failed to copy the files (err = %d)\n" $rv  \
                                        >> ${PD_TEMP_ERRLOG_FILE}
#
# We will NOT exit here but will save the time stamp so we will not
# attempt to re-red this again -- if we do, we will end up the same
# problem likd this during next boot.
#

fi
if [ -f $MISC_FILE ]
then
	/bin/cp $MISC_FILE  ${PD_DIR}/misc.pd$DATE
else
	printf "'$MISC_FILE' was missing - ksyms was not saved?\n" >  ${PD_DIR}/misc.pd$DATE
fi

#
# we will nullyfy the original file now
# The actual ksyms, etc will be copied in pdul_misc_post.sh
#
# let us create this file, voiding any previous contents
printf "\nCreated At:`/bin/date`\n" >  $MISC_FILE
# after creating such a file, we have to do sync to save the fs
/bin/sync
/bin/sync
/bin/sync # three times!? why???

#
# Everything copied successfully! So, let us save this time stamp
# so next time we will not attempt to copy again
# NOTE:
#	This time-stamp copying should be done at last
#
printf "
# This time stamp created on `/bin/date`
# with startup time in seconds: $DATE
#
$mtd_ts
#
" > $PD_CTRL_FILE

# Let is stamp this information also on the /mnt, so when we switch
# partition, we will NOT save the same dump on /mnt
#
if [ -d /mnt ]
then
printf "
# This time stamp created on `/bin/date`
# with startup time in seconds: $DATE
#
$mtd_ts
#
" > /mnt/$PD_CTRL_FILE
fi
sync
sync
sync
